/****************************************************************************************************************************************************
 * Copyright 2020 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 *    * Neither the name of the NXP. nor the names of
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************************************************************************************/

#include <FslGraphics/Sprite/Material/SpriteMaterialInfo.hpp>
#include <FslBase/Log/Math/Pixel/LogPxExtent2D.hpp>
#include <FslGraphics/UnitTest/Helper/Sprite/Material/Test/SpriteMaterialImpl.hpp>
#include <FslGraphics/UnitTest/Helper/TestFixtureFslGraphics.hpp>

using namespace Fsl;

namespace
{
  using TestSprite_SpriteMaterialInfo = TestFixtureFslGraphics;
}

TEST(TestSprite_SpriteMaterialInfo, Constuct)
{
  SpriteMaterialInfo info;
}

TEST(TestSprite_SpriteMaterialInfo, Constuct_Values)
{
  constexpr SpriteMaterialId id(12);
  constexpr PxExtent2D extent(42, 1337);
  constexpr uint32_t nativeMaterialFlags(127);
  auto spriteMaterial = std::make_shared<SpriteMaterialImpl>(id, extent);

  constexpr bool isOpaque = true;
  SpriteMaterialInfo info(id, extent, isOpaque, spriteMaterial, nativeMaterialFlags);

  EXPECT_TRUE(info.IsValid());
  EXPECT_EQ(id, info.Id);
  EXPECT_EQ(extent, info.ExtentPx);
  EXPECT_EQ(isOpaque, info.IsOpaque);
  EXPECT_EQ(spriteMaterial, info.Material);
  EXPECT_EQ(nativeMaterialFlags, info.NativeMaterialFlags);
}
